% -----------------------------------------------------------------------
%   COPYRIGHT (C) 2001-2022 MICHAEL ZRULL (TSIROULNIKOV). ALL RIGHTS RESERVED 
%   DIRECT OR INDIRECT COMMERCIAL DERIVATIVES OR USAGE PROHIBITED
%   GNU General Public License v.3+ <https://www.gnu.org/licenses/> 
% -----------------------------------------------------------------------
classdef fsaf_room
    %%
    properties
        cfgFPath='./';
        cfgFs=48000;
        cfgIrirLen % ms
        cfgDly % ms
        cfgHPF=50;
        cfgHPN=5;
        cfgLPF=500;
        cfgLPN=2;
        cfgfPre=6; % db/ms
        cfgfPost=3; % db/ms
        
        rir
        irir
    end
    methods
        %% ---------------------------------------------------------------------
        function o=fsaf_room(fpath,varargin)
            % fpath = path to working directory
            % varargin: config
            str=replace(fpath,'\\','/');
            o.cfgFPath=fpath;
            if str(length(str)) ~= '/'
                str=strcat(str,'/');
            end
            o.cfgFPath=str;
            if nargin > 1
                o=o.config(varargin{:});
            end
        end
        %% ---------------------------------------------------------------------
        function o = config(o,varargin)
            for n=1:2:nargin-1
                imq=varargin{n};
                param=varargin{n+1};
                if strcmpi(imq, 'rir')
                    if ischar(param)
                        fname=strcat(o.cfgFPath,param);
                        if isfile(fname)
                            [o.rir, o.CfgFs]=audioread(fname);
                        else
                            error('file %s does not exist',param);
                        end
                    else
                        o.rir=param;
                    end
                elseif strcmpi(imq, 'irirlen')
                    o.cfgIrirLen=param;
                elseif strcmpi(imq, 'dly')
                    o.cfgDly=param;
                elseif strcmpi(imq, 'hpf')
                    o.cfgHPF=param;
                elseif strcmpi(imq, 'hpn')
                    o.cfgHPN=param;
                elseif strcmpi(imq, 'lpf')
                    o.cfgLPF=param;
                elseif strcmpi(imq, 'lpn')
                    o.cfgLPN=param;
                elseif strcmpi(imq, 'pre')
                    o.cfgPre=param;
                elseif strcmpi(imq, 'post')
                    o.cfgPost=param;
                elseif strcmpi(imq, 'nse')
                    o.cfgNse=param;
                else
                end
            end
        end        
        %% ---------------------------------------------------------------------
        function o = rir_specgram_plot(o,figno,Wms)
            fs=o.cfgFs;
            if nargin < 3
                Wms=20;
            end
            W=Wms*fs;
            [S,ff,t]=spectrogram(ir,bohmanwin(W),3*W/4,fn/10,fs,'yaxis');
            S=db(S);
            S=S-max(max(S));
            S=max(S,-82);
            fsaf.fig(figno);
            mesh(t*1e3,ff,S,'FaceColor','flat');view([0 90]);
            grid on;
            colorbar
            set(gca,'YScale','log');
            title('SPK+RIR %.1fms spectrogram',Wms);
        end
        %% ---------------------------------------------------------------------
        function o = rir_fd_plot(o,figno)
            fs=o.cfgFs;
            [h,f]=freqz(o.rir,1,fs/2,fs);
            h=abs(h.*h');
            g=grpdelay(o.r,1,f,fs);
            g=g*1e3/fs;
            ga=mean(g(o.cfgHPF:o.cfgLPF));
            fsaf.fig(figno);
            ha=mean(h(o.cfgHPF:o.cfgLPF));
            h=db(h)/2;
            ha=db(ha)/2;
            h=h-ha;
            semilogx(f,h, o,-g);
            legend('FR','-GroupGly');
            xlim([20 fs/2]);
            ylim([-52 12]);
            str=sprintf('frequency response and group delay %.1f average',ga);
            title(str);            
            xlabel('frequyency, Hz');
            ylabel('level, dB | delay, ms');
        end
    end
end