% -----------------------------------------------------------------------
%   COPYRIGHT 2001-2022 MICHAEL ZRULL (TSIROULNIKOV). ALL RIGHTS RESERVED
%   DIRECT OR INDIRECT COMMERCIAL DERIVATIVES OR USAGE PROHIBITED
%   GNU General Public License v.3+ <https://www.gnu.org/licenses/>
% -----------------------------------------------------------------------
clear all
if ~contains(path(),'fsaf')
    addpath('/d/mlab/fsaf');
end
fs=48000;
fn=fs/2;
HPF=50;
HPN=5;
LPF=500;
LPN=2;
DLYs=40e-3;
LADFs=180e-3;
OFFs=LADFs/4;
RT60s=200e-3;
EPS=1e-4;
DIRECT=0.1;

if 0
    [hb,ha]=butter(HPN,HPF/fn,'high');
    [lb,la]=butter(LPN,LPF/fn);
    lenin=fs*RT60s;
    hx=impz(hb,ha,lenin);
    lx=impz(lb,la,lenin);
    fx=conv(hx,lx);
    fx=fx(1:lenin);
    t=1e3*(1:lenin)/fs;
    fsaf.td_plot(1,'time','hx',hx,'lx',lx,'fx',fx,'tnorm',fs*1e-3);
    %grid on;
    xlim([0 10])
    fsaf.fd_plot(2,'freq resp','hx',hx,'lx',lx,'fx',fx,'log',0.1,'m',24000);
    xlim([1 fn]);
    fsaf.fig(3);
    g=grpdelay(hx,1,fn,fs);
    semilogx(g*1e3/fs);
    grid on
    ylabel('delay,ms');
    xlim([1 fn]);
    fsaf.tddb_plot(4,'time','hx',hx,'lx',lx,'fx',fx,'tnorm',fs*1e-3);
end
tic
lenin=fs*LADFs;
d=ones(lenin,1);
idx=OFFs*fs;
dropre=1e-3^(1/idx);
dropost=1e-3^(1/(lenin-idx));
for k=idx-1:-1:1
    d(k)=d(k+1)*dropre;
end
for k=idx+1:lenin
    d(k)=d(k-1)*dropost;
end
[~,P0]=fsaf.rir_jtf(d,RT60s*fs);%,conv(hb,lb),conv(ha,la));
[rir,w]=fsaf.rir_make(RT60s*fs,RT60s*fs*2,DLYs*fs,0,0);
rir(DLYs*fs+1)=sum(w)*DIRECT;rir=rir/norm(rir);


%fsaf.fig(5);imagesc(db(P0));grid on;
%fsaf.fig(6);dd=db(diag(P0));plot(t,dd-max(dd),t,db(d));grid on
X=zeros(lenin*2-1,lenin);
x=zeros(1,lenin);
for k=1:lenin*2-1
%    x=circshift(x,1,2);
    x(2:end)=x(1:end-1);
%    x(1:end-1)=x(2:end);
%        x(end)=rir(k);
    x(1)=rir(k);
    X(k,:)=x;
end
toc
%tic
b=zeros(lenin*2-1,1);
%b(lenin+fs*(DLYs+OFFs))=1;
b(round(fs*(DLYs+OFFs)))=1;
%b(lenin+lenin/4)=1;
IP=fsaf.invS(P0,-120,2);
XX=X'*X;
nxx=trace(XX)/lenin;
Xb=X'*b;
toc
%tic
cor=linsolve(XX+EPS*nxx*IP,Xb);
toc
%h=flipud(h);
%%
cir=filter(cor,1,rir);
dcir=flt90(cir);
ccir=dcir(max(0,(OFFs-2e-3))*fs+1:end);
mx=max(ccir);
drir=flt90(rir);
mx=max(mx,max(drir));
ccir=ccir-mx;
drir=drir-mx;
dcor=flt90(cor);
%dlt=ccir-drir(1:length(ccir));
%
str=sprintf('RIR inversion for EPS=%.1e',EPS);
fsaf.fd_plot(4,str,'rir',rir,'inverted rir',cor,'corrected',cir,'m',24);ylim([-22 12]);
TSFT=DLYs;
fsaf.td_plot  (5,str,'rir',rir+.1,'inverted rir',cor,'corrected',cir-0.1,...
    'tnorm',fs,'xalign',-1,'tshift',TSFT);
xlim([-TSFT 2*RT60s-TSFT]);
xlabel('time, sec');
%fsaf.tddb_plot(6,str,'rir',nrir,'irir',cor,'conv',ncir,'xalign',-1);
fsaf.td_plot(6,str,'rir',drir,'inverted rir',dcor,...
    'corrected and realigned',ccir,...
    'tnorm',fs,'xalign',-1,'tshift',TSFT);
xlim([-TSFT 2*RT60s-TSFT]);
ylim([-72 2]);
ylabel('normalized level, dB');
xlabel('time, sec');
%
return

%% ---------------------------------------------------------------------
function y=flt90(x,LEN)
    if nargin < 2
        LEN=21;
    end
    OFF=(LEN-1)/2;
    x=db(x+eps);
    y=zeros(size(x));
    lenin=length(x);
    for k=OFF+1:lenin-OFF
        a=x(k-OFF:k+OFF);
        b=sort(a);
        if x(k) > b(LEN-1)+6
            y(k)=x(k);
        else
            y(k)=b(LEN-1);
        end
    end
    y(1:OFF)=y(OFF+1);
    y(lenin-OFF+1:lenin)=y(lenin-OFF);
end



