% -----------------------------------------------------------------------
%   COPYRIGHT 2001-2024 MICHAEL ZRULL (TSIROULNIKOV). ALL RIGHTS RESERVED
%   DIRECT OR INDIRECT COMMERCIAL DERIVATIVES OR USAGE PROHIBITED
%   GNU General Public License v.3+ <https://www.gnu.org/licenses/>
% -----------------------------------------------------------------------
clear all
if ~contains(path(),'fsaf')
    addpath('/d/mlab/fsaf');
end
fs=48000;
fn=fs/2;
HPF=1000;
HPN=1;
LPF=14000;
LPN=5;
DLYs=20e-3;
LADFs=20e-3;
OFFs=5e-3;
RT60s=150e-3;
EPS=1e-8;
DIRECT=0.1;
%%
[b,a]=butter(2,150/fn,'high');
[hb,ha]=butter(HPN,HPF/fn,'high');
hb=conv(hb,b);
ha=conv(ha,a);
[lb,la]=butter(LPN,LPF/fn);
lenin=fs*RT60s;
hx=impz(hb,ha,lenin);
lx=impz(lb,la,lenin);
fx=conv(hx,lx);
fx=fx(1:lenin); % thsiis the desired IR
tic
%%
load 'rir.mat';
[~,idx]=max(abs(ir));
idx=idx-DLYs*fs;
rir=ir(idx+1:end)*10^(-5/20);
lenin=floor(length(rir)/2)-1;
lenin=min(lenin, RT60s*fs);
%% the kernel-based ReLS inversion of IR
d=ones(lenin,1);
idx=OFFs*fs;
dropre=1e-3^(1/idx);
dropost=1e-3^(1/(lenin-idx));
for k=idx-1:-1:1
    d(k)=d(k+1)*dropre;
end
for k=idx+1:lenin
    d(k)=d(k-1)*dropost;
end
[~,P0]=fsaf.rir_jtf(d,RT60s*fs);
iri=cat(1,zeros(DLYs*fs,1),fx);
iri=iri(1:lenin);
X=zeros(lenin*2-1,lenin);
x=zeros(1,lenin);
for k=1:lenin*2-1
    x(2:end)=x(1:end-1);
    x(1)=rir(k);
    X(k,:)=x;
end
b=zeros(lenin*2-1,1);
st=round(fs*(DLYs+OFFs));
b(st+1:st+lenin)=iri;
toc % end of preparation, now making regularization matrix and LS
IP=fsaf.invS(P0,-120,2);
XX=X'*X;
nxx=trace(XX)/lenin;
Xb=X'*b;
toc % ReLS ready 
cor=linsolve(XX+EPS*nxx*IP,Xb);
toc % done
%% the resuts 
cir=filter(cor,1,rir);
irix=iri(fs*1e-3*2+1:end);
corx=cor(fs*1e-3*3+1:end);
cirx=cir(fs*1e-3*21+1:end);
diri=flt90(irix);
drir=flt90(rir);
dcor=flt90(corx);
dcir=flt90(cirx);
str=sprintf('RIR inversion for EPS=%.1e',EPS);
fsaf.fd_plot(1,str,'desired',irix,'rir',rir,'inverted rir',corx,'corrected',cirx,'m',24,'log',0.02);
ylim([-27 7]);
xlabel('frequency, kHz');
TSFT=DLYs;
fsaf.td_plot  (2,str,'desired',irix+0.2,'rir',rir+.1,'inverted rir',corx,'corrected',cirx-0.1,...
    'tnorm',fs,'xalign',-1,'tshift',TSFT);
xlim([-48*2.5/fs 48*6/fs]);
xlabel('time, sec');
%fsaf.tddb_plot(6,str,'rir',nrir,'irir',cor,'conv',ncir,'xalign',-1);
fsaf.td_plot(3,str,'desired',diri,'rir',drir,'inverted rir',dcor,...
    'corrected and realigned',dcir,...
    'tnorm',fs,'xalign',-1,'tshift',TSFT);
xlim([-TSFT TSFT*3]);
ylim([-110 0]);
ylabel('normalized level, dB');
xlabel('time, sec');
return

%% ---------------------------------------------------------------------
% 90th quantil filtering to decease ripples for plotting
function y=flt90(x,LEN)
    if nargin < 2
        LEN=21;
    end
    OFF=(LEN-1)/2;
    x=db(x+eps);
    y=zeros(size(x));
    lenin=length(x);
    for k=OFF+1:lenin-OFF
        a=x(k-OFF:k+OFF);
        b=sort(a);
        if x(k) > b(LEN-1)+6
            y(k)=x(k);
        else
            y(k)=b(LEN-1);
        end
    end
    y(1:OFF)=y(OFF+1);
    y(lenin-OFF+1:lenin)=y(lenin-OFF);
end



